// Include gulp.
const gulp = require('gulp');

// Include gulp plugins.
const babel = require('gulp-babel');
const eslint = require('gulp-eslint');
const rename = require('gulp-rename');
const uglifyEs = require('gulp-uglify-es').default;

// Paths and sources used in the theme.
const paths = {
  js: {
    dir: 'js',
    ext: 'js',
  },
};

/**
 * Runs ESLint on all js files.
 *
 * @async
 * @return {object} stream.
 */
function lintJs() {
  const src = `./assets/${paths.js.dir}/**/*.${paths.js.ext}`;

  return gulp.src([src])
    .pipe(eslint())
    .pipe(eslint.format())
    .pipe(eslint.failAfterError());
}
lintJs.description = 'Lints all JS files.';

/**
 * Compiles js files with babel.
 *
 * @async
 * @return {object} stream.
 */
function compileJs() {
  const src = `./assets/${paths.js.dir}/**/*.${paths.js.ext}`;
  const dest = `./assets/dist/${paths.js.dir}`;

  return gulp.src(src, '!./**/*.min.*')
    .pipe(babel({
      presets: ['@babel/env'],
    }))
    .pipe(gulp.dest(dest));
}
compileJs.description = 'Compiles JS files using Babel.';

/**
 * Minifies JS files and saves them with the .min.js extension.
 *
 * @async
 * @return {object} stream.
 */
function minifyJs() {
  const src = `./assets/dist/${paths.js.dir}/**/*.${paths.js.ext}`;
  const dest = `./assets/dist/${paths.js.dir}`;

  return gulp.src([src, '!./**/*.min.*'])
    .pipe(uglifyEs())
    .pipe(rename({
      suffix: '.min',
    }))
    .pipe(gulp.dest(dest));
}
minifyJs.description = 'Minifies all JS files.';

/**
 * Moves all required libraries to lib directory.
 *
 * @async
 * @return {object} stream.
 */
function loadLibs() {
  const libraries = [
    './node_modules/@babel/polyfill/dist/polyfill.min.js',
  ];
  const dest = `./assets/dist/${paths.js.dir}`;

  return gulp.src(libraries)
    .pipe(gulp.dest(dest));
}
minifyJs.description = 'Moves all required libraries to lib directory.';

/**
 * Runs default script tasks.
 *
 * @param {Object} cb Call back.
 */
function scripts(cb) {
  gulp.series(
    lintJs,
    compileJs,
    minifyJs,
  )(cb);
}
scripts.description = 'Lints all JS files, compiles them, and minifies them.';

/**
 * Runs default lib tasks.
 * @param {Object} cb Call back.
 */
function libs(cb) {
  gulp.series(
    loadLibs,
  )(cb);
}
libs.description = 'Loads and moves all libraries to lib directory.';

/**
 * Runs scripting tasks depending on which file has changed.
 */
function watch() {
  gulp.watch([`./assets/${paths.js.dir}/**/*.${paths.js.ext}`, '!./**/*.min.*'], scripts);
}
watch.description = 'Watches all JS files and runs scripts depending on what has been altered.';

/**
 * Compiles and minifies all JS files.
 *
 * @param {Object} cb Call back.
 */
function build(cb) {
  gulp.series(
    compileJs,
    minifyJs,
    loadLibs,
  )(cb);
}
build.description = 'Compiles and minifies all JS files and loads libraries.';

/**
 * Runs default scripting tasks.
 *
 * @param {Object} cb Call back.
 */
function defaultTask(cb) {
  gulp.series(
    scripts,
    libs,
  )(cb);
}
defaultTask.description = 'Lints all JS files, compiles them, minifies them, and loads libraries.';

// Default task.
gulp.task('default', defaultTask);

// Build task.
gulp.task(build);

// Watch task.
gulp.task(watch);

// Scripting tasks.
gulp.task(scripts);
gulp.task('scripts:lint', lintJs);
gulp.task('scripts:minify', minifyJs);
gulp.task('scripts:compile', compileJs);

// Lib tasks.
gulp.task(libs);
